package ru.dbotthepony.mc.otm.datagen.advancements

import net.minecraft.advancements.Advancement
import net.minecraft.advancements.AdvancementRewards
import net.minecraft.advancements.FrameType
import net.minecraft.advancements.RequirementsStrategy
import net.minecraft.advancements.critereon.InventoryChangeTrigger
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.data.ExistingFileHelper
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.datagen.lang.MatteryLanguageProvider
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MRegistry
import ru.dbotthepony.mc.otm.triggers.BlackHoleTrigger
import java.util.function.Consumer

fun addAdvancements(serializer: Consumer<Advancement>, existingFileHelper: ExistingFileHelper, lang: MatteryLanguageProvider) {
	val translation = lang.english.Prepended("otm.advancements.regular")

	val root = AdvancementBuilder()
		.requirements(RequirementsStrategy.OR)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_INGOT),
			title = TranslatableComponent(translation.add("root", "Overdrive That Matters")),
			description = TranslatableComponent(translation.add("root.desc", "Its all about things that matter")),
			showToast = false,
			announceChat = false,
			background = modLocation("textures/block/decorative/tritanium_block_gray.png")
		)
		.addCriterion("has_tritanium_ore", criterion(MItemTags.TRITANIUM_ORES))
		.addCriterion("has_tritanium_ore_clump", criterion(MItemTags.TRITANIUM_ORE_CLUMPS))
		.addCriterion("has_tritanium_ingot", InventoryChangeTrigger.TriggerInstance.hasItems(MItems.TRITANIUM_INGOT))
		.save(serializer, modLocation("regular/root"), existingFileHelper)

	val crude = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.BATTERY_CRUDE),
			title = TranslatableComponent(translation.add("crude_battery", "Potato Power!")),
			description = TranslatableComponent(translation.add("crude_battery.desc", "Put together a Crude Battery. Better hope for getter better power source soon")),
		)
		.addCriterion("has_item", criterion(MItems.BATTERY_CRUDE))
		.save(serializer, modLocation("regular/crude_battery"), existingFileHelper)

	val normal = AdvancementBuilder()
		.parent(crude)
		.display(
			itemStack = ItemStack(MItems.BATTERY_NORMAL),
			title = TranslatableComponent(translation.add("normal_battery", "Power Storage")),
			description = TranslatableComponent(translation.add("normal_battery.desc", "Put together an Ordinary Battery")),
		)
		.addCriterion("has_item", criterion(MItems.BATTERY_NORMAL))
		.save(serializer, modLocation("regular/normal_battery"), existingFileHelper)

	AdvancementBuilder()
		.parent(normal)
		.display(
			itemStack = ItemStack(MItems.BATTERY_DENSE),
			title = TranslatableComponent(translation.add("dense_battery", "Extra Space Battery")),
			description = TranslatableComponent(translation.add("dense_battery.desc", "Put together a Dense Battery. Better not to expose it to fires!")),
		)
		.addCriterion("has_item", criterion(MItems.BATTERY_DENSE))
		.save(serializer, modLocation("regular/dense_battery"), existingFileHelper)

	val capacitor = AdvancementBuilder()
		.parent(normal)
		.display(
			itemStack = ItemStack(MItems.BATTERY_CAPACITOR),
			title = TranslatableComponent(translation.add("capacitor_battery", "Supercapacitor")),
			description = TranslatableComponent(translation.add("capacitor_battery.desc", "Put together a Capacitor Battery. Surely, you gonna need them somewhere...")),
		)
		.addCriterion("has_item", criterion(MItems.BATTERY_CAPACITOR))
		.save(serializer, modLocation("regular/capacitor_battery"), existingFileHelper)

	AdvancementBuilder()
		.parent(capacitor)
		.display(
			itemStack = ItemStack(MItems.ENERGY_SWORD),
			title = TranslatableComponent(translation.add("energy_sword", "Self Sharpening Blade")),
			description = TranslatableComponent(translation.add("energy_sword.desc", "Wield a High-Frequency Blade, a melee weapon intended to slice Creepers into creep-cakes. Rational folks won't attempt to slice their Android fellows...")),
			frameType = FrameType.GOAL
		)
		.addCriterion("has_item", criterion(MItems.ENERGY_SWORD))
		.save(serializer, modLocation("regular/energy_sword"), existingFileHelper)

	AdvancementBuilder()
		.parent(normal)
		.display(
			itemStack = ItemStack(MItems.QUANTUM_BATTERY),
			title = TranslatableComponent(translation.add("quantum_battery", "Power, in Superposition")),
			description = TranslatableComponent(translation.add("quantum_battery.desc", "Put together a Quantum Battery, powered by Ender technologies")),
			frameType = FrameType.GOAL
		)
		.rewards(AdvancementRewards.Builder.experience(50))
		.requirements(RequirementsStrategy.OR)
		.addCriterion("has_item0", criterion(MItems.QUANTUM_BATTERY))
		.addCriterion("has_item1", criterion(MItems.QUANTUM_CAPACITOR))
		.save(serializer, modLocation("regular/quantum_battery"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.ZPM_BATTERY),
			title = TranslatableComponent(translation.add("zpm_battery", "Pocket Universe, as Power Source")),
			description = TranslatableComponent(translation.add("zpm_battery.desc", "Find Zero Point Module, something from different universe of ours, created using technologies lost in time in all possible multiverses")),
			frameType = FrameType.CHALLENGE,
			hidden = true
		)
		.rewards(AdvancementRewards.Builder.experience(800))
		.addCriterion("has_item", criterion(MItems.ZPM_BATTERY))
		.save(serializer, modLocation("regular/zpm_battery"), existingFileHelper)

	val blackhole = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.BLACK_HOLE),
			title = TranslatableComponent(translation.add("black_hole_pull", "Something Massive and Something Close")),
			description = TranslatableComponent(translation.add("black_hole_pull.desc", "Experience Singularity's gravitational force, better not to get closer")),
			hidden = true
		)
		.addCriterion("pulled_by_black_hole", BlackHoleTrigger.Instance)
		.save(serializer, modLocation("regular/black_hole"), existingFileHelper)

	AdvancementBuilder()
		.parent(blackhole)
		.display(
			itemStack = ItemStack(MItems.BLACK_HOLE_SCANNER),
			title = TranslatableComponent(translation.add("black_hole_scanner", "Determining the Mass")),
			description = TranslatableComponent(translation.add("black_hole_scanner.desc", "Craft the Singularity Scanner, to determine mass of Singularity from safe distance")),
		)
		.addCriterion("has_item", criterion(MItems.BLACK_HOLE_SCANNER))
		.save(serializer, modLocation("regular/black_hole_scanner"), existingFileHelper)

	AdvancementBuilder()
		.parent(blackhole)
		.display(
			itemStack = ItemStack(MItems.GRAVITATION_STABILIZER),
			title = TranslatableComponent(translation.add("stabilizer", "Reducing the Impact")),
			description = TranslatableComponent(translation.add("stabilizer.desc", "Put together a device that defy physical laws and also defy gravity of Singularities. Better hope it does not cause any side effects")),
		)
		.addCriterion("has_item", criterion(MItems.GRAVITATION_STABILIZER))
		.save(serializer, modLocation("regular/stabilizer"), existingFileHelper)

	AdvancementBuilder()
		.parent(blackhole)
		.display(
			itemStack = ItemStack(MItems.PORTABLE_GRAVITATION_STABILIZER),
			title = TranslatableComponent(translation.add("portable_stabilizer", "Local Gravity Field")),
			description = TranslatableComponent(translation.add("portable_stabilizer.desc", "Protect yourself from gravitational effect of Singularity using Portable Gravitation Stabilizer")),
		)
		.addCriterion("has_item", criterion(MItems.PORTABLE_GRAVITATION_STABILIZER))
		.save(serializer, modLocation("regular/portable_stabilizer"), existingFileHelper)

	val ore = AdvancementBuilder()
		.parent(root)
		.requirements(RequirementsStrategy.OR)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_ORE_CLUMP),
			title = TranslatableComponent(translation.add("ore", "Blue Metal Discovery")),
			description = TranslatableComponent(translation.add("ore.desc", "Mine some Tritanium")),
		)
		.addCriterion("has_tritanium_ore", criterion(MItemTags.TRITANIUM_ORES))
		.addCriterion("has_tritanium_ore_clump", criterion(MItemTags.TRITANIUM_ORE_CLUMPS))
		.save(serializer, modLocation("regular/ore"), existingFileHelper)

	val ingot = AdvancementBuilder()
		.parent(ore)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_INGOT),
			title = TranslatableComponent(translation.add("ingot", "Acquire Harder-ware")),
			description = TranslatableComponent(translation.add("ingot.desc", "Smelt a Tritanium ingot")),
		)
		.addCriterion("has_tritanium_ingot", criterion(MItemTags.TRITANIUM_INGOTS))
		.save(serializer, modLocation("regular/ingot"), existingFileHelper)

	AdvancementBuilder()
		.parent(ingot)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_PICKAXE),
			title = TranslatableComponent(translation.add("pickaxe", "A Tool for Patient Miners")),
			description = TranslatableComponent(translation.add("pickaxe.desc", "Craft a Tritanium Pickaxe")),
		)
		.addCriterion("has_tritanium_pickaxe", criterion(MItems.TRITANIUM_PICKAXE))
		.save(serializer, modLocation("regular/pickaxe"), existingFileHelper)

	AdvancementBuilder()
		.parent(ingot)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_HOE),
			title = TranslatableComponent(translation.add("hoe", "A Tool for Farmers")),
			description = TranslatableComponent(translation.add("hoe.desc", "Tritanium is a very good choice for making a sturdy Hoe")),
			hidden = true
		)
		.addCriterion("hoe", criterion(MItems.TRITANIUM_HOE))
		.save(serializer, modLocation("regular/hoe"), existingFileHelper)

	val plate = AdvancementBuilder()
		.parent(ingot)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_PLATE),
			title = TranslatableComponent(translation.add("plate", "Hard Plating")),
			description = TranslatableComponent(translation.add("plate.desc", "Roll down some Tritanium using a Plate Press"))
		)
		.addCriterion("has_item", criterion(MItemTags.TRITANIUM_PLATES))
		.save(serializer, modLocation("regular/plate"), existingFileHelper)

	AdvancementBuilder()
		.parent(plate)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_CHESTPLATE),
			title = TranslatableComponent(translation.add("armor", "Composite Armor")),
			description = TranslatableComponent(translation.add("armor.desc", "Bend some Tritanium plates into simple yet sturdy armor"))
		)
		.requirements(RequirementsStrategy.OR)
		.addCriterion("has_item0", criterion(MItems.TRITANIUM_HELMET))
		.addCriterion("has_item1", criterion(MItems.TRITANIUM_CHESTPLATE))
		.addCriterion("has_item2", criterion(MItems.TRITANIUM_PANTS))
		.addCriterion("has_item3", criterion(MItems.TRITANIUM_BOOTS))
		.save(serializer, modLocation("regular/armor"), existingFileHelper)

	val glass = AdvancementBuilder()
		.parent(plate)
		.display(
			itemStack = ItemStack(MRegistry.INDUSTRIAL_GLASS.item),
			title = TranslatableComponent(translation.add("industrial_glass", "Extra Hard Glass")),
			description = TranslatableComponent(translation.add("industrial_glass.desc", "Manual says it should be bulletproof."))
		)
		.requirements(RequirementsStrategy.OR)
		.also { advancement ->
			MRegistry.INDUSTRIAL_GLASS.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/industrial_glass"), existingFileHelper)

	AdvancementBuilder()
		.parent(glass)
		.display(
			itemStack = ItemStack(MRegistry.INDUSTRIAL_GLASS.getItem(DyeColor.GREEN)),
			title = TranslatableComponent(translation.add("industrial_glass2", "Glass-tacular Artist")),
			description = TranslatableComponent(translation.add("industrial_glass2.desc", "Paint Industrial Glass all possible colors")),
			frameType = FrameType.GOAL
		)
		.requirements(RequirementsStrategy.AND)
		.also { advancement ->
			MRegistry.INDUSTRIAL_GLASS.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/industrial_glass2"), existingFileHelper)

	val cargoCrate = AdvancementBuilder()
		.parent(plate)
		.display(
			itemStack = ItemStack(MRegistry.CARGO_CRATES.item),
			title = TranslatableComponent(translation.add("cargo_crate", "Sturdy Item Stash")),
			description = TranslatableComponent(translation.add("cargo_crate.desc", "Cargo Crates, like Double Chest, but Single."))
		)
		.requirements(RequirementsStrategy.OR)
		.also { advancement ->
			MRegistry.CARGO_CRATES.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/cargo_crate"), existingFileHelper)

	val cargoCrateInMinecart = AdvancementBuilder()
		.parent(cargoCrate)
		.display(
			itemStack = ItemStack(MItems.CARGO_CRATE_MINECARTS[null]!!),
			title = TranslatableComponent(translation.add("cargo_crate_minecart", "Crate On a Rail")),
			description = TranslatableComponent(translation.add("cargo_crate_minecart.desc", "Drop a Cargo Crate onto Minecart and see how it goes"))
		)
		.requirements(RequirementsStrategy.OR)
		.also { advancement ->
			MItems.CARGO_CRATE_MINECARTS.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/cargo_crate_minecart"), existingFileHelper)

	AdvancementBuilder()
		.parent(cargoCrateInMinecart)
		.display(
			itemStack = ItemStack(MItems.CARGO_CRATE_MINECARTS[DyeColor.GREEN]!!),
			title = TranslatableComponent(translation.add("cargo_crate_minecart2", "A Motley Train")),
			description = TranslatableComponent(translation.add("cargo_crate_minecart2.desc", "Have all color variants of Minecarts with Cargo Crates")),
			frameType = FrameType.GOAL
		)
		.requirements(RequirementsStrategy.AND)
		.also { advancement ->
			MItems.CARGO_CRATE_MINECARTS.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/cargo_crate_minecart2"), existingFileHelper)

	AdvancementBuilder()
		.parent(cargoCrate)
		.display(
			itemStack = ItemStack(MRegistry.CARGO_CRATES.item),
			title = TranslatableComponent(translation.add("cargo_crate2", "Colorful Warehouse")),
			description = TranslatableComponent(translation.add("cargo_crate2.desc", "Craft all color variants of Cargo Crates")),
			frameType = FrameType.GOAL
		)
		.requirements(RequirementsStrategy.AND)
		.also { advancement ->
			MRegistry.CARGO_CRATES.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/cargo_crate2"), existingFileHelper)

	val tritaniumBlock = AdvancementBuilder()
		.parent(plate)
		.display(
			itemStack = ItemStack(MRegistry.TRITANIUM_BLOCK.item),
			title = TranslatableComponent(translation.add("tritanium_block", "Cold, Impregnable Wall")),
			description = TranslatableComponent(translation.add("tritanium_block.desc", "Coat stones in Tritanium Plates, a cheap yet incredibly sturdy material"))
		)
		.requirements(RequirementsStrategy.OR)
		.also { advancement ->
			MRegistry.TRITANIUM_BLOCK.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STRIPED_BLOCK.flatItems.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STAIRS.allItems.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STRIPED_STAIRS.flatItems.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MItems.TRITANIUM_STRIPED_BLOCK.also { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MItems.TRITANIUM_STRIPED_STAIRS.also { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/tritanium_block"), existingFileHelper)

	AdvancementBuilder()
		.parent(tritaniumBlock)
		.display(
			itemStack = ItemStack(MItems.TRITANIUM_STRIPED_BLOCK),
			title = TranslatableComponent(translation.add("striped_tritanium_block", "Old Fashion Color Touch")),
			description = TranslatableComponent(translation.add("striped_tritanium_block.desc", "Pale Blue coat with Yellow stripe, I bet you know whose design is this"))
		)
		.requirements(RequirementsStrategy.OR)
		.addCriterion("has_item", criterion(MItems.TRITANIUM_STRIPED_BLOCK))
		.addCriterion("has_item1", criterion(MItems.TRITANIUM_STRIPED_STAIRS))
		.addCriterion("has_item2", criterion(MItems.TRITANIUM_STRIPED_SLAB))
		.addCriterion("has_item3", criterion(MItems.TRITANIUM_STRIPED_WALL))
		.save(serializer, modLocation("regular/striped_tritanium_block"), existingFileHelper)

	val colorTritaniumBlock = AdvancementBuilder()
		.parent(tritaniumBlock)
		.display(
			itemStack = ItemStack(MRegistry.TRITANIUM_BLOCK.getItem(DyeColor.GREEN)),
			title = TranslatableComponent(translation.add("tritanium_block2", "Colorful Fortress")),
			description = TranslatableComponent(translation.add("tritanium_block2.desc", "Put some paint over Tritanium Block to make it look fabulous"))
		)
		.requirements(RequirementsStrategy.OR)
		.also { advancement ->
			MRegistry.TRITANIUM_BLOCK.items.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STAIRS.items.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STRIPED_BLOCK.flatItems.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STRIPED_STAIRS.flatItems.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MItems.TRITANIUM_STRIPED_BLOCK.also { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MItems.TRITANIUM_STRIPED_STAIRS.also { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/tritanium_block2"), existingFileHelper)

	val colorfulTritaniumBlock = AdvancementBuilder()
		.parent(colorTritaniumBlock)
		.display(
			itemStack = ItemStack(MRegistry.TRITANIUM_BLOCK.getItem(DyeColor.BLACK)),
			title = TranslatableComponent(translation.add("tritanium_block3", "Paint Me A Castle")),
			description = TranslatableComponent(translation.add("tritanium_block3.desc", "Craft all color variants of Tritanium Blocks")),
			frameType = FrameType.GOAL
		)
		.rewards(AdvancementRewards.Builder.loot(modLocation("tritanium_block3")).addExperience(100))
		.requirements(RequirementsStrategy.AND)
		.also { advancement ->
			MRegistry.TRITANIUM_BLOCK.items.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/tritanium_block3"), existingFileHelper)

	AdvancementBuilder()
		.parent(colorfulTritaniumBlock)
		.display(
			itemStack = ItemStack(MRegistry.TRITANIUM_STRIPED_BLOCK.getItem(DyeColor.BLACK, DyeColor.WHITE)),
			title = TranslatableComponent(translation.add("tritanium_block4", "All The Colors")),
			description = TranslatableComponent(translation.add("tritanium_block4.desc", "Craft ALL color variants of Tritanium Blocks including striped ones")),
			frameType = FrameType.CHALLENGE
		)
		.rewards(AdvancementRewards.Builder.loot(modLocation("tritanium_block4")).addExperience(400))
		.requirements(RequirementsStrategy.AND)
		.also { advancement ->
			MRegistry.TRITANIUM_BLOCK.items.values.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MRegistry.TRITANIUM_STRIPED_BLOCK.flatItems.forEach { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
			MItems.TRITANIUM_STRIPED_BLOCK.also { advancement.addCriterion(it.registryName!!.path, criterion(it)) }
		}
		.save(serializer, modLocation("regular/tritanium_block4"), existingFileHelper)

	val pill = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.PILL_ANDROID),
			title = TranslatableComponent(translation.add("pill", "Side Colored Mystery")),
			description = TranslatableComponent(translation.add("pill.desc", "Find some of those mysterious pills. Consult with Cleric before trying to ingest them.")),
		)
		.requirements(RequirementsStrategy.OR)
		.addCriterion("pill1", criterion(MItems.PILL_ANDROID))
		.addCriterion("pill2", criterion(MItems.PILL_HEAL))
		.addCriterion("pill3", criterion(MItems.PILL_HUMANE))
		.addCriterion("pill4", criterion(MItems.PILL_OBLIVION))
		.save(serializer, modLocation("regular/pill"), existingFileHelper)

	AdvancementBuilder()
		.parent(pill)
		.display(
			itemStack = ItemStack(MItems.PILL_HEAL),
			title = TranslatableComponent(translation.add("all_pills", "Take Your Meds")),
			description = TranslatableComponent(translation.add("all_pills.desc", "Find all possible pill types")),
			frameType = FrameType.CHALLENGE,
			hidden = true
		)
		.rewards(AdvancementRewards.Builder.experience(200))
		.requirements(RequirementsStrategy.AND)
		.addCriterion("pill1", criterion(MItems.PILL_ANDROID))
		.addCriterion("pill2", criterion(MItems.PILL_HEAL))
		.addCriterion("pill3", criterion(MItems.PILL_HUMANE))
		.addCriterion("pill4", criterion(MItems.PILL_OBLIVION))
		.save(serializer, modLocation("regular/all_pills"), existingFileHelper)
}
