package ru.dbotthepony.mc.otm.item

import net.minecraft.ChatFormatting
import net.minecraft.network.chat.Component
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.TooltipFlag
import net.minecraft.world.level.Level
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.matter.IMatterItem
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.matter.IMatterValue
import ru.dbotthepony.mc.otm.matter.MatterValue

class MatterDustItem : Item(Properties().tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB).stacksTo(64)), IMatterItem {
	private fun matter(stack: ItemStack): ImpreciseFraction {
		return stack.tag?.get("matter")?.let { return@let ImpreciseFraction.deserializeNBT(it) } ?: return ImpreciseFraction.ZERO
	}

	private fun matter(stack: ItemStack, matter: ImpreciseFraction) {
		stack.orCreateTag["matter"] = matter.serializeNBT()
	}

	override fun getMatterValue(stack: ItemStack): IMatterValue? {
		val value = stack.tag?.get("matter") ?: return null
		return MatterValue(ImpreciseFraction.deserializeNBT(value), 1.0)
	}

	override fun canDecompose(stack: ItemStack) = false

	override fun appendHoverText(
		p_41421_: ItemStack,
		p_41422_: Level?,
		p_41423_: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		super.appendHoverText(p_41421_, p_41422_, p_41423_, p_41424_)
		val matter = this.getMatterValue(p_41421_)

		if (matter == null) {
			p_41423_.add(DESC)
		}

		p_41423_.add(DESC2)

		if (matter != null) {
			p_41423_.add(DESC3)
		}
	}

	fun addMatterValue(stack: ItemStack, matter: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
		if (stack.count != 1)
			return ImpreciseFraction.ZERO

		val matterThis = matter(stack)

		if (matterThis >= MAX_MATTER_IN_ITEM)
			return ImpreciseFraction.ZERO

		val newMatter = (matterThis + matter).coerceAtMost(MAX_MATTER_IN_ITEM)
		val diff = newMatter - matterThis

		if (!simulate)
			matter(stack, newMatter)

		return diff
	}

	fun isFull(stack: ItemStack): Boolean {
		return matter(stack) >= MAX_MATTER_IN_ITEM
	}

	companion object {
		private val DESC = TranslatableComponent("item.overdrive_that_matters.matter_dust.desc").withStyle(ChatFormatting.DARK_GRAY)
		private val DESC2 = TranslatableComponent("item.overdrive_that_matters.matter_dust.desc2").withStyle(ChatFormatting.GRAY)
		private val DESC3 = TranslatableComponent("item.overdrive_that_matters.matter_dust.desc3").withStyle(ChatFormatting.DARK_GRAY)
		val MAX_MATTER_IN_ITEM = ImpreciseFraction(4)
	}
}
